@@ -25,7 +25,7 @@ from .nsiswriter import NSISFileWriter
 from .wheels import WheelGetter
 from .util import download, get_cache_dir, normalize_path
 
-__version__ = '2.6'
+__version__ = '2.7'
 
 pjoin = os.path.join
 logger = logging.getLogger(__name__)
@@ -234,7 +234,7 @@ class InstallerBuilder(object):
     SCRIPT_TEMPLATE = """#!python{qualifier}
 import sys, os
 import site
-scriptdir, script = os.path.split(__file__)
+scriptdir, script = os.path.split(os.path.abspath(__file__))
 installdir = scriptdir  # for compatibility with commands
 pkgdir = os.path.join(scriptdir, 'pkgs')
 sys.path.insert(0, pkgdir)
